package com.bcxin.cacahe.controller;

import cn.myapps.base.web.WebUser;
import cn.myapps.common.controller.Resource;
import cn.myapps.common.util.StringUtil;
import cn.myapps.common.util.cache.ICacheProvider;
import cn.myapps.common.util.cache.IMyCache;
import cn.myapps.common.util.cache.MyCacheManager;
import com.bcxin.saas.core.components.DataPermissionScopeConfig;
import com.bcxin.saas.core.components.DistributedCacheProvider;
import com.bcxin.saas.core.components.dtos.DataPermissionType;
import com.bcxin.saas.core.exceptions.SaasBadException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.util.Collection;
import java.util.HashSet;
import java.util.stream.Collectors;

@RestController
@RequestMapping(value = {"/api/caches"})
public class CacheController {
    private static final Logger logger = LoggerFactory.getLogger(CacheController.class);

    private final DistributedCacheProvider distributedCacheProvider;

    public CacheController(DistributedCacheProvider distributedCacheProvider) {
        this.distributedCacheProvider = distributedCacheProvider;
    }

    @PostMapping("/{cacheType}/refresh")
    public Resource refresh(@PathVariable String cacheType, @RequestBody Collection<String> ids) {
        if (StringUtil.isBlank(cacheType)) {
            throw new SaasBadException("缓存类型不能为空");
        }
        if (CollectionUtils.isEmpty(ids)) {
            throw new SaasBadException("待清除的Id信息不能为空");
        }


        int affectedCount = 0;
        String cacheTypeValue = cacheType.toLowerCase().trim();
        switch (cacheTypeValue) {
            case "user": {
                ICacheProvider provider = MyCacheManager.getProviderInstance(WebUser.CACHE_PROVIDER);
                for (String empId : ids) {
                    IMyCache iMyCache = provider.getCache(WebUser.CACHE_KEY_WEBUSER);
                    if (iMyCache != null) {
                        iMyCache.remove(empId);
                        affectedCount++;
                    }
                }

                logger.error("自动清除本地缓存:{}", affectedCount, ids.stream().collect(Collectors.joining(",")));
                break;
            }
            case "depart-permissions":
            case "security-permissions":
            case "org-permissions": {
                Collection<String> cacheKeys = new HashSet<>();
                for (String selId : ids) {
                    String cacheKey = DataPermissionScopeConfig.getPermissionScopeCacheKey(DataPermissionType.Enterprise, selId);
                    if (cacheTypeValue == "depart-permissions") {
                        cacheKey = DataPermissionScopeConfig.getPermissionScopeCacheKey(DataPermissionType.Department, selId);
                    } else if (cacheTypeValue == "security-permissions") {
                        cacheKey = DataPermissionScopeConfig.getPermissionScopeCacheKey(DataPermissionType.Securtity, selId);
                    }

                    cacheKeys.add(cacheKey);
                }

                this.distributedCacheProvider.remove(cacheKeys);
                break;
            }

        }

        Resource resource = new Resource(affectedCount, null, ids, null);
        return resource;
    }
}
